==> create-fly-1.21.8-6.0.8-1-sources.jar: com/zurrtum/create/client/content/kinetics/steamEngine/SteamEngineVisual.java <==
package com.zurrtum.create.client.content.kinetics.steamEngine;

import com.zurrtum.create.catnip.math.AngleHelper;
import com.zurrtum.create.client.AllPartialModels;
import com.zurrtum.create.client.content.kinetics.base.KineticBlockEntityRenderer;
import com.zurrtum.create.client.flywheel.api.instance.Instance;
import com.zurrtum.create.client.flywheel.api.visual.DynamicVisual;
import com.zurrtum.create.client.flywheel.api.visualization.VisualizationContext;
import com.zurrtum.create.client.flywheel.lib.instance.InstanceTypes;
import com.zurrtum.create.client.flywheel.lib.instance.TransformedInstance;
import com.zurrtum.create.client.flywheel.lib.model.Models;
import com.zurrtum.create.client.flywheel.lib.visual.AbstractBlockEntityVisual;
import com.zurrtum.create.client.flywheel.lib.visual.SimpleDynamicVisual;
import com.zurrtum.create.content.kinetics.steamEngine.PoweredShaftBlockEntity;
import com.zurrtum.create.content.kinetics.steamEngine.SteamEngineBlock;
import com.zurrtum.create.content.kinetics.steamEngine.SteamEngineBlockEntity;
import java.util.Objects;
import java.util.function.Consumer;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_3532;

public class SteamEngineVisual extends AbstractBlockEntityVisual<SteamEngineBlockEntity> implements SimpleDynamicVisual {

    protected final TransformedInstance piston;
    protected final TransformedInstance linkage;
    protected final TransformedInstance connector;

    private Float lastAngle = Float.NaN;
    private class_2351 lastAxis = null;

    public SteamEngineVisual(VisualizationContext context, SteamEngineBlockEntity blockEntity, float partialTick) {
        super(context, blockEntity, partialTick);

        piston = instancerProvider().instancer(InstanceTypes.TRANSFORMED, Models.partial(AllPartialModels.ENGINE_PISTON)).createInstance();
        linkage = instancerProvider().instancer(InstanceTypes.TRANSFORMED, Models.partial(AllPartialModels.ENGINE_LINKAGE)).createInstance();
        connector = instancerProvider().instancer(InstanceTypes.TRANSFORMED, Models.partial(AllPartialModels.ENGINE_CONNECTOR)).createInstance();

        animate();
    }

    @Override
    public void beginFrame(DynamicVisual.Context ctx) {
        animate();
    }

    private void animate() {
        Float angle = SteamEngineRenderer.getTargetAngle(blockEntity);
        class_2351 axis = class_2351.field_11052;

        PoweredShaftBlockEntity shaft = blockEntity.getShaft();
        if (shaft != null)
            axis = KineticBlockEntityRenderer.getRotationAxisOf(shaft);

        if (Objects.equals(angle, lastAngle) && lastAxis == axis) {
            return;
        }

        lastAngle = angle;
        lastAxis = axis;

        if (angle == null) {
            piston.setVisible(false);
            linkage.setVisible(false);
            connector.setVisible(false);
            return;
        } else {
            piston.setVisible(true);
            linkage.setVisible(true);
            connector.setVisible(true);
        }

        class_2350 facing = SteamEngineBlock.getFacing(blockState);
        class_2351 facingAxis = facing.method_10166();

        boolean roll90 = facingAxis.method_10179() && axis == class_2351.field_11052 || facingAxis.method_10178() && axis == class_2351.field_11051;
        float piston = ((6 / 16f) * class_3532.method_15374(angle) - class_3532.method_15355(class_3532.method_27285(14 / 16f) - class_3532.method_27285(6 / 16f) * class_3532.method_27285(
            class_3532.method_15362(angle))));
        float distance = class_3532.method_15355(class_3532.method_27285(piston - 6 / 16f * class_3532.method_15374(angle)));
        float angle2 = (float) Math.acos(distance / (14 / 16f)) * (class_3532.method_15362(angle) >= 0 ? 1f : -1f);

        transformed(this.piston, facing, roll90).translate(0, piston + 20 / 16f, 0).setChanged();

        transformed(linkage, facing, roll90).center().translate(0, 1, 0).uncenter().translate(0, piston + 20 / 16f, 0).translate(0, 4 / 16f, 8 / 16f)
            .rotateX(angle2).translate(0, -4 / 16f, -8 / 16f).setChanged();

        transformed(connector, facing, roll90).translate(0, 2, 0).center().rotateX(-(angle + class_3532.field_29845)).uncenter().setChanged();
    }

    protected TransformedInstance transformed(TransformedInstance modelData, class_2350 facing, boolean roll90) {
        return modelData.setIdentityTransform().translate(getVisualPosition()).center().rotateYDegrees(AngleHelper.horizontalAngle(facing))
            .rotateXDegrees(AngleHelper.verticalAngle(facing) + 90).rotateYDegrees(roll90 ? -90 : 0).uncenter();
    }

    @Override
    public void updateLight(float partialTick) {
        relight(piston, linkage, connector);
    }

    @Override
    protected void _delete() {
        piston.delete();
        linkage.delete();
        connector.delete();
    }

    @Override
    public void collectCrumblingInstances(Consumer<Instance> consumer) {
        consumer.accept(piston);
        consumer.accept(linkage);
        consumer.accept(connector);
    }
}

==> create-fly-1.21.11-6.0.8-2-sources.jar: com/zurrtum/create/client/content/kinetics/steamEngine/SteamEngineVisual.java <==
package com.zurrtum.create.client.content.kinetics.steamEngine;

import com.zurrtum.create.catnip.math.AngleHelper;
import com.zurrtum.create.client.AllPartialModels;
import com.zurrtum.create.client.content.kinetics.base.KineticBlockEntityRenderer;
import com.zurrtum.create.client.flywheel.api.instance.Instance;
import com.zurrtum.create.client.flywheel.api.visual.DynamicVisual;
import com.zurrtum.create.client.flywheel.api.visualization.VisualizationContext;
import com.zurrtum.create.client.flywheel.lib.instance.InstanceTypes;
import com.zurrtum.create.client.flywheel.lib.instance.TransformedInstance;
import com.zurrtum.create.client.flywheel.lib.model.Models;
import com.zurrtum.create.client.flywheel.lib.visual.AbstractBlockEntityVisual;
import com.zurrtum.create.client.flywheel.lib.visual.SimpleDynamicVisual;
import com.zurrtum.create.content.kinetics.steamEngine.PoweredShaftBlockEntity;
import com.zurrtum.create.content.kinetics.steamEngine.SteamEngineBlock;
import com.zurrtum.create.content.kinetics.steamEngine.SteamEngineBlockEntity;

import java.util.Objects;
import java.util.function.Consumer;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_3532;

public class SteamEngineVisual extends AbstractBlockEntityVisual<SteamEngineBlockEntity> implements SimpleDynamicVisual {

    protected final TransformedInstance piston;
    protected final TransformedInstance linkage;
    protected final TransformedInstance connector;

    private Float lastAngle = Float.NaN;
    private class_2351 lastAxis = null;

    public SteamEngineVisual(VisualizationContext context, SteamEngineBlockEntity blockEntity, float partialTick) {
        super(context, blockEntity, partialTick);

        piston = instancerProvider().instancer(InstanceTypes.TRANSFORMED, Models.partial(AllPartialModels.ENGINE_PISTON)).createInstance();
        linkage = instancerProvider().instancer(InstanceTypes.TRANSFORMED, Models.partial(AllPartialModels.ENGINE_LINKAGE)).createInstance();
        connector = instancerProvider().instancer(InstanceTypes.TRANSFORMED, Models.partial(AllPartialModels.ENGINE_CONNECTOR)).createInstance();

        animate();
    }

    @Override
    public void beginFrame(DynamicVisual.Context ctx) {
        animate();
    }

    private void animate() {
        Float angle = SteamEngineRenderer.getTargetAngle(blockEntity);
        class_2351 axis = class_2351.field_11052;

        PoweredShaftBlockEntity shaft = blockEntity.getShaft();
        if (shaft != null)
            axis = KineticBlockEntityRenderer.getRotationAxisOf(shaft);

        if (Objects.equals(angle, lastAngle) && lastAxis == axis) {
            return;
        }

        lastAngle = angle;
        lastAxis = axis;

        if (angle == null) {
            piston.setVisible(false);
            linkage.setVisible(false);
            connector.setVisible(false);
            return;
        } else {
            piston.setVisible(true);
            linkage.setVisible(true);
            connector.setVisible(true);
        }

        class_2350 facing = SteamEngineBlock.getFacing(blockState);
        class_2351 facingAxis = facing.method_10166();

        boolean roll90 = facingAxis.method_10179() && axis == class_2351.field_11052 || facingAxis.method_10178() && axis == class_2351.field_11051;
        float piston = ((6 / 16f) * class_3532.method_15374(angle) - class_3532.method_15355(class_3532.method_27285(14 / 16f) - class_3532.method_27285(6 / 16f) * class_3532.method_27285(class_3532.method_15362(angle))));
        float distance = class_3532.method_15355(class_3532.method_27285(piston - 6 / 16f * class_3532.method_15374(angle)));
        float angle2 = (float) Math.acos(distance / (14 / 16f)) * (class_3532.method_15362(angle) >= 0 ? 1f : -1f);

        transformed(this.piston, facing, roll90).translate(0, piston + 20 / 16f, 0).setChanged();

        transformed(linkage, facing, roll90).center().translate(0, 1, 0).uncenter().translate(0, piston + 20 / 16f, 0).translate(0, 4 / 16f, 8 / 16f)
            .rotateX(angle2).translate(0, -4 / 16f, -8 / 16f).setChanged();

        transformed(connector, facing, roll90).translate(0, 2, 0).center().rotateX(-(angle + class_3532.field_29845)).uncenter().setChanged();
    }

    protected TransformedInstance transformed(TransformedInstance modelData, class_2350 facing, boolean roll90) {
        return modelData.setIdentityTransform().translate(getVisualPosition()).center().rotateYDegrees(AngleHelper.horizontalAngle(facing))
            .rotateXDegrees(AngleHelper.verticalAngle(facing) + 90).rotateYDegrees(roll90 ? -90 : 0).uncenter();
    }

    @Override
    public void updateLight(float partialTick) {
        relight(piston, linkage, connector);
    }

    @Override
    protected void _delete() {
        piston.delete();
        linkage.delete();
        connector.delete();
    }

    @Override
    public void collectCrumblingInstances(Consumer<Instance> consumer) {
        consumer.accept(piston);
        consumer.accept(linkage);
        consumer.accept(connector);
    }
}
